@@ -4,23 +4,6 @@ BlackWidow Ultimate 2013 effects
 from openrazer_daemon.dbus_services import endpoint
 
 
-@endpoint('razer.device.lighting.bw2013', 'getEffect', out_sig='y')
-def bw_get_effect(self):
-    """
-    Get current effect
-
-    :return: Brightness
-    :rtype: int
-    """
-    self.logger.debug("DBus call bw_get_effect")
-
-    driver_path = self.get_driver_path('matrix_effect_pulsate')
-
-    with open(driver_path, 'r') as driver_file:
-        brightness = int(driver_file.read().strip())
-        return brightness
-
-
 @endpoint('razer.device.lighting.bw2013', 'setPulsate')
 def bw_set_pulsate(self):
     """
@@ -30,6 +13,9 @@ def bw_set_pulsate(self):
 
     driver_path = self.get_driver_path('matrix_effect_pulsate')
 
+    # remember effect
+    self.set_persistence("backlight", "effect", 'pulsate')
+
     with open(driver_path, 'w') as driver_file:
         driver_file.write('1')
 
@@ -46,6 +32,9 @@ def bw_set_static(self):
 
     driver_path = self.get_driver_path('matrix_effect_static')
 
+    # remember effect
+    self.set_persistence("backlight", "effect", 'static')
+
     with open(driver_path, 'w') as driver_file:
         driver_file.write('1')
 
