@@ -20,6 +20,10 @@ def set_logo_static_naga_hex_v2(self, red, green, blue):
     # Notify others
     self.send_effect_event('setStatic', red, green, blue)
 
+    # remember effect
+    self.set_persistence("logo", "effect", 'static')
+    self.zone["logo"]["colors"][0:3] = int(red), int(green), int(blue)
+
     rgb_driver_path = self.get_driver_path('logo_matrix_effect_static')
 
     payload = bytes([red, green, blue])
@@ -38,6 +42,9 @@ def set_logo_spectrum_naga_hex_v2(self):
     # Notify others
     self.send_effect_event('setSpectrum')
 
+    # remember effect
+    self.set_persistence("logo", "effect", 'spectrum')
+
     effect_driver_path = self.get_driver_path('logo_matrix_effect_spectrum')
 
     with open(effect_driver_path, 'w') as effect_driver_file:
@@ -54,6 +61,9 @@ def set_logo_none_naga_hex_v2(self):
     # Notify others
     self.send_effect_event('setNone')
 
+    # remember effect
+    self.set_persistence("logo", "effect", 'none')
+
     driver_path = self.get_driver_path('logo_matrix_effect_none')
 
     with open(driver_path, 'w') as driver_file:
@@ -84,6 +94,11 @@ def set_logo_reactive_naga_hex_v2(self, red, green, blue, speed):
     # Notify others
     self.send_effect_event('setReactive', red, green, blue, speed)
 
+    # remember effect
+    self.set_persistence("logo", "effect", 'reactive')
+    self.zone["logo"]["colors"][0:3] = int(red), int(green), int(blue)
+    self.set_persistence("logo", "speed", int(speed))
+
     if speed not in (1, 2, 3, 4):
         speed = 4
 
@@ -103,6 +118,9 @@ def set_logo_breath_random_naga_hex_v2(self):
     # Notify others
     self.send_effect_event('setBreathRandom')
 
+    # remember effect
+    self.set_persistence("logo", "effect", 'breathRandom')
+
     driver_path = self.get_driver_path('logo_matrix_effect_breath')
 
     payload = b'1'
@@ -130,6 +148,10 @@ def set_logo_breath_single_naga_hex_v2(self, red, green, blue):
     # Notify others
     self.send_effect_event('setBreathSingle', red, green, blue)
 
+    # remember effect
+    self.set_persistence("logo", "effect", 'breathSingle')
+    self.zone["logo"]["colors"][0:3] = int(red), int(green), int(blue)
+
     driver_path = self.get_driver_path('logo_matrix_effect_breath')
 
     payload = bytes([red, green, blue])
@@ -166,6 +188,10 @@ def set_logo_breath_dual_naga_hex_v2(self, red1, green1, blue1, red2, green2, bl
     # Notify others
     self.send_effect_event('setBreathDual', red1, green1, blue1, red2, green2, blue2)
 
+    # remember effect
+    self.set_persistence("logo", "effect", 'breathDual')
+    self.zone["logo"]["colors"][0:6] = int(red1), int(green1), int(blue1), int(red2), int(green2), int(blue2)
+
     driver_path = self.get_driver_path('logo_matrix_effect_breath')
 
     payload = bytes([red1, green1, blue1, red2, green2, blue2])
@@ -193,6 +219,10 @@ def set_scroll_static_naga_hex_v2(self, red, green, blue):
     # Notify others
     self.send_effect_event('setStatic', red, green, blue)
 
+    # remember effect
+    self.set_persistence("scroll", "effect", 'static')
+    self.zone["scroll"]["colors"][0:3] = int(red), int(green), int(blue)
+
     rgb_driver_path = self.get_driver_path('scroll_matrix_effect_static')
 
     payload = bytes([red, green, blue])
@@ -211,6 +241,8 @@ def set_scroll_spectrum_naga_hex_v2(self):
     # Notify others
     self.send_effect_event('setSpectrum')
 
+    self.set_persistence("scroll", "effect", 'spectrum')
+
     effect_driver_path = self.get_driver_path('scroll_matrix_effect_spectrum')
 
     with open(effect_driver_path, 'w') as effect_driver_file:
@@ -227,6 +259,8 @@ def set_scroll_none_naga_hex_v2(self):
     # Notify others
     self.send_effect_event('setNone')
 
+    self.set_persistence("scroll", "effect", 'none')
+
     driver_path = self.get_driver_path('scroll_matrix_effect_none')
 
     with open(driver_path, 'w') as driver_file:
@@ -257,6 +291,11 @@ def set_scroll_reactive_naga_hex_v2(self, red, green, blue, speed):
     # Notify others
     self.send_effect_event('setReactive', red, green, blue, speed)
 
+    # remember effect
+    self.set_persistence("scroll", "effect", 'reactive')
+    self.zone["scroll"]["colors"][0:3] = int(red), int(green), int(blue)
+    self.set_persistence("scroll", "speed", int(speed))
+
     if speed not in (1, 2, 3, 4):
         speed = 4
 
@@ -276,6 +315,9 @@ def set_scroll_breath_random_naga_hex_v2(self):
     # Notify others
     self.send_effect_event('setBreathRandom')
 
+    # remember effect
+    self.set_persistence("scroll", "effect", 'breathRandom')
+
     driver_path = self.get_driver_path('scroll_matrix_effect_breath')
 
     payload = b'1'
@@ -303,6 +345,10 @@ def set_scroll_breath_single_naga_hex_v2(self, red, green, blue):
     # Notify others
     self.send_effect_event('setBreathSingle', red, green, blue)
 
+    # remember effect
+    self.set_persistence("scroll", "effect", 'breathSingle')
+    self.zone["scroll"]["colors"][0:3] = int(red), int(green), int(blue)
+
     driver_path = self.get_driver_path('scroll_matrix_effect_breath')
 
     payload = bytes([red, green, blue])
@@ -339,6 +385,10 @@ def set_scroll_breath_dual_naga_hex_v2(self, red1, green1, blue1, red2, green2,
     # Notify others
     self.send_effect_event('setBreathDual', red1, green1, blue1, red2, green2, blue2)
 
+    # remember effect
+    self.set_persistence("scroll", "effect", 'breathDual')
+    self.zone["scroll"]["colors"][0:6] = int(red1), int(green1), int(blue1), int(red2), int(green2), int(blue2)
+
     driver_path = self.get_driver_path('scroll_matrix_effect_breath')
 
     payload = bytes([red1, green1, blue1, red2, green2, blue2])
