@@ -29,62 +29,30 @@ def copy_func(function_reference, name=None):
         return types.FunctionType(function_reference.__code__, function_reference.__globals__, name or function_reference.func_name, function_reference.__defaults__, function_reference.__closure__)
 
 
-class DBusServiceFactory(object):
-    """
-    Factory object to create different service objects.
-
-    Different service objects are useful, as the DBus service table is stored in the class which would be shared
-    between all class instances.
-    """
-    service_number = 0
-
-    @staticmethod
-    def get_service(bus_name, object_path):
-        """
-        Get an instance of the service history
-
-        :param bus_name: DBus bus name
-        :type bus_name: str
-
-        :param object_path: DBus Object name
-        :type object_path: str
-
-        :return: New object
-        :rtype: DBusService
-        """
-        new_service = type("DBUSService{0:04}".format(DBusServiceFactory.service_number), (DBusService,), {})
-        DBusServiceFactory.service_number += 1
-
-        return new_service(bus_name, object_path)
-
-
 class DBusService(dbus.service.Object):
     """
     DBus Service object
 
     Allows for dynamic method adding
     """
-    BUS_TYPE = 'session'
+    BUS_NAME = 'org.razer'
 
-    def __init__(self, bus_name, object_path):
+    def __init__(self, object_path):
         """
         Init the object
 
-        :param bus_name: DBus bus name
-        :type bus_name: str
-
         :param object_path: DBus Object name
         :type object_path: str
         """
-        self.bus_name = bus_name
-        self.object_path = object_path
+        # We could pass (bus, object_path) here, but we rather register the object manually.
+        super().__init__()
 
-        if DBusService.BUS_TYPE == 'session':
-            bus_object = dbus.service.BusName(bus_name, bus=dbus.SessionBus())
-        else:
-            bus_object = dbus.service.BusName(bus_name, bus=dbus.SystemBus())
+        bus = dbus.SessionBus()
 
-        super(DBusService, self).__init__(bus_object, object_path)
+        # the constructor of BusName registers the bus, the returned object is not used but must be kept
+        self.bus_name_obj = dbus.service.BusName(self.BUS_NAME, bus)
+
+        self.add_to_connection(bus, object_path)
 
     def add_dbus_method(self, interface_name, function_name, function, in_signature=None, out_signature=None, byte_arrays=False):
         """
