@@ -107,6 +107,9 @@ struct razer_report razer_chroma_misc_set_dock_charge_type(unsigned char charge_
 struct razer_report razer_chroma_misc_get_polling_rate(void);
 struct razer_report razer_chroma_misc_set_polling_rate(unsigned short polling_rate);
 
+struct razer_report razer_chroma_misc_get_polling_rate2(void);
+struct razer_report razer_chroma_misc_set_polling_rate2(unsigned short polling_rate);
+
 struct razer_report razer_chroma_misc_get_dock_brightness(void);
 struct razer_report razer_chroma_misc_set_dock_brightness(unsigned char brightness);
 
@@ -116,6 +119,9 @@ struct razer_report razer_chroma_misc_get_dpi_xy(unsigned char variable_storage)
 struct razer_report razer_chroma_misc_set_dpi_xy_byte(unsigned char dpi_x,unsigned char dpi_y);
 struct razer_report razer_chroma_misc_get_dpi_xy_byte(void);
 
+struct razer_report razer_chroma_misc_set_dpi_stages(unsigned char variable_storage, unsigned char count, unsigned char active_stage, const unsigned short *dpi);
+struct razer_report razer_chroma_misc_get_dpi_stages(unsigned char variable_storage);
+
 struct razer_report razer_chroma_misc_get_idle_time(void);
 struct razer_report razer_chroma_misc_set_idle_time(unsigned short idle_time);
 
