@@ -21,6 +21,7 @@
 #define USB_DEVICE_ID_RAZER_ORBWEAVER 0x0113
 #define USB_DEVICE_ID_RAZER_BLACKWIDOW_ULTIMATE_2013 0x011A
 #define USB_DEVICE_ID_RAZER_BLACKWIDOW_STEALTH 0x011B
+#define USB_DEVICE_ID_RAZER_BLACKWIDOW_TE_2014 0x011C
 #define USB_DEVICE_ID_RAZER_TARTARUS 0x0201
 #define USB_DEVICE_ID_RAZER_DEATHSTALKER_EXPERT 0x0202
 #define USB_DEVICE_ID_RAZER_BLACKWIDOW_CHROMA 0x0203
@@ -47,6 +48,7 @@
 #define USB_DEVICE_ID_RAZER_BLACKWIDOW_ELITE 0x0228
 #define USB_DEVICE_ID_RAZER_CYNOSA_CHROMA 0x022A
 #define USB_DEVICE_ID_RAZER_TARTARUS_V2 0x022B
+#define USB_DEVICE_ID_RAZER_CYNOSA_CHROMA_PRO 0x022C
 #define USB_DEVICE_ID_RAZER_BLADE_STEALTH_MID_2017 0x022D
 #define USB_DEVICE_ID_RAZER_BLADE_PRO_2017_FULLHD 0x022F
 #define USB_DEVICE_ID_RAZER_BLADE_STEALTH_LATE_2017 0x0232
@@ -66,10 +68,26 @@
 #define USB_DEVICE_ID_RAZER_BLADE_STEALTH_LATE_2019 0x024A
 #define USB_DEVICE_ID_RAZER_BLADE_PRO_LATE_2019 0x024C
 #define USB_DEVICE_ID_RAZER_BLADE_STUDIO_EDITION_2019 0x024D
+#define USB_DEVICE_ID_RAZER_BLACKWIDOW_V3 0x024E
 #define USB_DEVICE_ID_RAZER_BLADE_STEALTH_EARLY_2020 0x0252
 #define USB_DEVICE_ID_RAZER_BLADE_15_ADV_2020 0x0253
 #define USB_DEVICE_ID_RAZER_BLADE_EARLY_2020_BASE 0x0255
+#define USB_DEVICE_ID_RAZER_BLADE_PRO_EARLY_2020 0x0256
+#define USB_DEVICE_ID_RAZER_HUNTSMAN_MINI 0x0257
+#define USB_DEVICE_ID_RAZER_BLACKWIDOW_V3_MINI 0x0258
+#define USB_DEVICE_ID_RAZER_BLADE_STEALTH_LATE_2020 0x0259
+#define USB_DEVICE_ID_RAZER_BLACKWIDOW_V3_PRO_WIRED 0x025A
+#define USB_DEVICE_ID_RAZER_ORNATA_V2 0x025D
 #define USB_DEVICE_ID_RAZER_CYNOSA_V2 0x025E
+#define USB_DEVICE_ID_RAZER_HUNTSMAN_V2_ANALOG 0x0266
+#define USB_DEVICE_ID_RAZER_BOOK_2020 0x026A
+#define USB_DEVICE_ID_RAZER_BLADE_15_ADV_EARLY_2021 0x026D
+#define USB_DEVICE_ID_RAZER_BLADE_15_BASE_EARLY_2021 0x026F
+#define USB_DEVICE_ID_RAZER_BLADE_14_2021 0x0270
+#define USB_DEVICE_ID_RAZER_BLACKWIDOW_V3_MINI_WIRELESS 0x0271
+#define USB_DEVICE_ID_RAZER_BLADE_15_ADV_MID_2021 0x0276
+#define USB_DEVICE_ID_RAZER_BLADE_17_PRO_MID_2021 0x0279
+#define USB_DEVICE_ID_RAZER_BLACKWIDOW_V3_TK 0x0A24
 
 /* Each keyboard report has 90 bytes*/
 #define RAZER_BLACKWIDOW_REPORT_LEN 0x5A
@@ -112,6 +130,9 @@
 #define RAZER_BLACKWIDOW_CHROMA_WAIT_MIN_US 600
 #define RAZER_BLACKWIDOW_CHROMA_WAIT_MAX_US 800
 
+#define RAZER_BLACKWIDOW_V3_WIRELESS_WAIT_MIN_US 4900
+#define RAZER_BLACKWIDOW_V3_WIRELESS_WAIT_MAX_US 5000
+
 #define RAZER_FIREFLY_WAIT_MIN_US 900
 #define RAZER_FIREFLY_WAIT_MAX_US 1000
 
