@@ -46,6 +46,7 @@
 #define USB_DEVICE_ID_RAZER_LANCEHEAD_TE_WIRED 0x0060
 #define USB_DEVICE_ID_RAZER_ATHERIS_RECEIVER 0x0062
 #define USB_DEVICE_ID_RAZER_BASILISK 0x0064
+#define USB_DEVICE_ID_RAZER_BASILISK_ESSENTIAL 0x0065
 #define USB_DEVICE_ID_RAZER_NAGA_TRINITY 0x0067
 #define USB_DEVICE_ID_RAZER_ABYSSUS_ELITE_DVA_EDITION 0x006A
 #define USB_DEVICE_ID_RAZER_ABYSSUS_ESSENTIAL 0x006B
@@ -59,9 +60,18 @@
 #define USB_DEVICE_ID_RAZER_VIPER 0x0078
 #define USB_DEVICE_ID_RAZER_VIPER_ULTIMATE_WIRED 0x007A
 #define USB_DEVICE_ID_RAZER_VIPER_ULTIMATE_WIRELESS 0x007B
+#define USB_DEVICE_ID_RAZER_DEATHADDER_V2_PRO_WIRED 0x007C
+#define USB_DEVICE_ID_RAZER_DEATHADDER_V2_PRO_WIRELESS 0x007D
 #define USB_DEVICE_ID_RAZER_BASILISK_X_HYPERSPEED 0x0083
 #define USB_DEVICE_ID_RAZER_DEATHADDER_V2 0x0084
+#define USB_DEVICE_ID_RAZER_BASILISK_V2 0x0085
+#define USB_DEVICE_ID_RAZER_BASILISK_ULTIMATE_WIRED 0x0086
+#define USB_DEVICE_ID_RAZER_BASILISK_ULTIMATE_RECEIVER 0x0088
 #define USB_DEVICE_ID_RAZER_VIPER_MINI 0x008A
+#define USB_DEVICE_ID_RAZER_DEATHADDER_V2_MINI 0x008C
+#define USB_DEVICE_ID_RAZER_NAGA_LEFT_HANDED_2020 0x008D
+#define USB_DEVICE_ID_RAZER_VIPER_8K 0x0091
+#define USB_DEVICE_ID_RAZER_DEATHADDER_ESSENTIAL_2021 0x0098
 
 /* Each keyboard report has 90 bytes*/
 #define RAZER_REPORT_LEN 0x5A
@@ -82,9 +92,21 @@
 #define RAZER_VIPER_MOUSE_RECEIVER_WAIT_MIN_US 59900
 #define RAZER_VIPER_MOUSE_RECEIVER_WAIT_MAX_US 60000
 
+#define RAZER_MOUSE_MAX_DPI_STAGES 5
+
 struct razer_mouse_device {
     struct usb_device *usb_dev;
     struct mutex lock;
+
+    struct input_dev *input;
+    struct hrtimer repeat_timer;
+    unsigned int tilt_hwheel;
+    unsigned int tilt_repeat_delay;
+    unsigned int tilt_repeat;
+    __s32 hwheel_value;
+    u8 button_byte; // Previous value of mouse button byte in HID record
+    u8 rep4[16]; // Previous value of report 4 on the keyboard intf
+
     unsigned char usb_interface_protocol;
     unsigned char usb_interface_subclass;
 
