@@ -51,8 +51,24 @@ class RazerDevice(object):
             'serial': True,
             'brightness': self._has_feature('razer.device.lighting.brightness'),
 
+            'battery': self._has_feature('razer.device.power', 'getBattery'),
+            'get_idle_time': self._has_feature('razer.device.power', 'getIdleTime'),
+            'set_idle_time': self._has_feature('razer.device.power', 'setIdleTime'),
+            'get_low_battery_threshold': self._has_feature('razer.device.power', 'getLowBatteryThreshold'),
+            'set_low_battery_threshold': self._has_feature('razer.device.power', 'setLowBatteryThreshold'),
+
             'macro_logic': self._has_feature('razer.device.macro'),
             'keyboard_layout': self._has_feature('razer.device.misc', 'getKeyboardLayout'),
+            'game_mode_led': self._has_feature('razer.device.led.gamemode'),
+            'macro_mode_led': self._has_feature('razer.device.led.macromode', 'setMacroMode'),
+            'macro_mode_led_effect': self._has_feature('razer.device.led.macromode', 'setMacroEffect'),
+            'macro_mode_modifier': self._has_feature('razer.device.macro', 'setModeModifier'),
+
+            'poll_rate': self._has_feature('razer.device.misc', ('getPollRate', 'setPollRate')),
+            'supported_poll_rates': self._has_feature('razer.device.misc', 'getSupportedPollRates'),
+            'dpi': self._has_feature('razer.device.dpi', ('getDPI', 'setDPI')),
+            'dpi_stages': self._has_feature('razer.device.dpi', ('getDPIStages', 'setDPIStages')),
+            'available_dpi': self._has_feature('razer.device.dpi', 'availableDPI'),
 
             # Default device is a chroma so lighting capabilities
             'lighting': self._has_feature('razer.device.lighting.chroma'),
@@ -138,6 +154,40 @@ class RazerDevice(object):
             'lighting_profile_led_red': self._has_feature('razer.device.lighting.profile_led', 'setRedLED'),
             'lighting_profile_led_green': self._has_feature('razer.device.lighting.profile_led', 'setGreenLED'),
             'lighting_profile_led_blue': self._has_feature('razer.device.lighting.profile_led', 'setBlueLED'),
+
+            # Charging Pad attrs
+            'lighting_charging': self._has_feature('razer.device.lighting.charging'),
+            'lighting_charging_active': self._has_feature('razer.device.lighting.charging', 'setChargingActive'),
+            'lighting_charging_brightness': self._has_feature('razer.device.lighting.charging', 'setChargingBrightness'),
+            'lighting_charging_spectrum': self._has_feature('razer.device.lighting.charging', 'setChargingSpectrum'),
+            'lighting_charging_static': self._has_feature('razer.device.lighting.charging', 'setChargingStatic'),
+            'lighting_charging_none': self._has_feature('razer.device.lighting.charging', 'setChargingNone'),
+            'lighting_charging_wave': self._has_feature('razer.device.lighting.charging', 'setChargingWave'),
+            'lighting_charging_breath_single': self._has_feature('razer.device.lighting.charging', 'setChargingBreathSingle'),
+            'lighting_charging_breath_dual': self._has_feature('razer.device.lighting.charging', 'setChargingBreathDual'),
+            'lighting_charging_breath_random': self._has_feature('razer.device.lighting.charging', 'setChargingBreathRandom'),
+
+            'lighting_fast_charging': self._has_feature('razer.device.lighting.fast_charging'),
+            'lighting_fast_charging_active': self._has_feature('razer.device.lighting.fast_charging', 'setFastChargingActive'),
+            'lighting_fast_charging_brightness': self._has_feature('razer.device.lighting.fast_charging', 'setFastChargingBrightness'),
+            'lighting_fast_charging_spectrum': self._has_feature('razer.device.lighting.fast_charging', 'setFastChargingSpectrum'),
+            'lighting_fast_charging_static': self._has_feature('razer.device.lighting.fast_charging', 'setFastChargingStatic'),
+            'lighting_fast_charging_none': self._has_feature('razer.device.lighting.fast_charging', 'setFastChargingNone'),
+            'lighting_fast_charging_wave': self._has_feature('razer.device.lighting.fast_charging', 'setFastChargingWave'),
+            'lighting_fast_charging_breath_single': self._has_feature('razer.device.lighting.fast_charging', 'setFastChargingBreathSingle'),
+            'lighting_fast_charging_breath_dual': self._has_feature('razer.device.lighting.fast_charging', 'setFastChargingBreathDual'),
+            'lighting_fast_charging_breath_random': self._has_feature('razer.device.lighting.fast_charging', 'setFastChargingBreathRandom'),
+
+            'lighting_fully_charged': self._has_feature('razer.device.lighting.fully_charged'),
+            'lighting_fully_charged_active': self._has_feature('razer.device.lighting.fully_charged', 'setFullyChargedActive'),
+            'lighting_fully_charged_brightness': self._has_feature('razer.device.lighting.fully_charged', 'setFullyChargedBrightness'),
+            'lighting_fully_charged_spectrum': self._has_feature('razer.device.lighting.fully_charged', 'setFullyChargedSpectrum'),
+            'lighting_fully_charged_static': self._has_feature('razer.device.lighting.fully_charged', 'setFullyChargedStatic'),
+            'lighting_fully_charged_none': self._has_feature('razer.device.lighting.fully_charged', 'setFullyChargedNone'),
+            'lighting_fully_charged_wave': self._has_feature('razer.device.lighting.fully_charged', 'setFullyChargedWave'),
+            'lighting_fully_charged_breath_single': self._has_feature('razer.device.lighting.fully_charged', 'setFullyChargedBreathSingle'),
+            'lighting_fully_charged_breath_dual': self._has_feature('razer.device.lighting.fully_charged', 'setFullyChargedBreathDual'),
+            'lighting_fully_charged_breath_random': self._has_feature('razer.device.lighting.fully_charged', 'setFullyChargedBreathRandom'),
         }
 
         # Nasty hack to convert dbus.Int32 into native
@@ -164,6 +214,17 @@ class RazerDevice(object):
         else:
             self.macro = None
 
+        if self.has('dpi'):
+            self._dbus_interfaces['dpi'] = _dbus.Interface(self._dbus, "razer.device.dpi")
+        if self.has('battery'):
+            self._dbus_interfaces['power'] = _dbus.Interface(self._dbus, "razer.device.power")
+        if self.has('game_mode_led'):
+            self._dbus_interfaces['game_mode_led'] = _dbus.Interface(self._dbus, "razer.device.led.gamemode")
+        if self.has('macro_mode_led'):
+            self._dbus_interfaces['macro_mode_led'] = _dbus.Interface(self._dbus, "razer.device.led.macromode")
+        if self.has('lighting_profile_led_red') or self.has('lighting_profile_led_green') or self.has('lighting_profile_led_blue'):
+            self._dbus_interfaces['profile_led'] = _dbus.Interface(self._dbus, "razer.device.lighting.profile_led")
+
     def _get_available_features(self):
         introspect_interface = _dbus.Interface(self._dbus, 'org.freedesktop.DBus.Introspectable')
         xml_spec = introspect_interface.Introspect()
@@ -358,6 +419,64 @@ class RazerDevice(object):
             "perspective_img": self.device_image
         }
 
+    @property
+    def battery_level(self) -> int:
+        """
+        Get battery level from device
+
+        :return: Battery level (0-100)
+        """
+        if self.has('battery'):
+            return int(self._dbus_interfaces['power'].getBattery())
+
+    @property
+    def is_charging(self) -> bool:
+        """
+        Get whether the device is charging or not
+
+        :return: Boolean
+        """
+        if self.has('battery'):
+            return bool(self._dbus_interfaces['power'].isCharging())
+
+    def set_idle_time(self, idle_time) -> None:
+        """
+        Sets the idle time on the device
+
+        :param idle_time: the time in seconds
+        """
+        if self.has('battery'):
+            self._dbus_interfaces['power'].setIdleTime(idle_time)
+
+    def get_idle_time(self) -> int:
+        """
+        Gets the idle time of the device
+
+        :return: Number of seconds before this device goes into powersave
+                 (60-900)
+        """
+        if self.has('battery'):
+            return int(self._dbus_interfaces['power'].getIdleTime())
+
+    def set_low_battery_threshold(self, threshold) -> None:
+        """
+        Set the low battery threshold as a percentage
+
+        :param threshold: Battery threshold as a percentage
+        :type threshold: int
+        """
+        if self.has('battery'):
+            self._dbus_interfaces['power'].setLowBatteryThreshold(threshold)
+
+    def get_low_battery_threshold(self) -> int:
+        """
+        Get the low battery threshold as a percentage
+
+        :return: Battery threshold as a percentage
+        """
+        if self.has('battery'):
+            return int(self._dbus_interfaces['power'].getLowBatteryThreshold())
+
     def __str__(self):
         return self._name
 
