@@ -50,7 +50,7 @@ class BaseRazerFX(object):
 
 class RazerFX(BaseRazerFX):
     def __init__(self, serial: str, capabilities: dict, daemon_dbus=None, matrix_dims=(-1, -1)):
-        super(RazerFX, self).__init__(serial, capabilities, daemon_dbus)
+        super().__init__(serial, capabilities, daemon_dbus)
 
         self._lighting_dbus = _dbus.Interface(self._dbus, "razer.device.lighting.chroma")
 
@@ -68,6 +68,46 @@ class RazerFX(BaseRazerFX):
 
         self.misc = MiscLighting(serial, capabilities, self._dbus)
 
+    @property
+    def effect(self) -> str:
+        """
+        Get current effect
+
+        :return: Effect name ("static", "spectrum", etc.)
+        :rtype: str
+        """
+        return self._lighting_dbus.getEffect()
+
+    @property
+    def colors(self) -> bytearray:
+        """
+        Get current effect colors
+
+        :return: Effect colors (an array of 9 bytes, for 3 colors in RGB format)
+        :rtype: bytearray
+        """
+        return bytes(self._lighting_dbus.getEffectColors())
+
+    @property
+    def speed(self) -> int:
+        """
+        Get current effect speed
+
+        :return: Effect speed (a value between 0 and 3)
+        :rtype: int
+        """
+        return self._lighting_dbus.getEffectSpeed()
+
+    @property
+    def wave_dir(self) -> int:
+        """
+        Get current wave direction
+
+        :return: Wave direction (WAVE_LEFT or WAVE_RIGHT)
+        :rtype: int
+        """
+        return self._lighting_dbus.getWaveDir()
+
     def none(self) -> bool:
         """
         No effect
@@ -551,7 +591,7 @@ class RazerFX(BaseRazerFX):
 
 class RazerAdvancedFX(BaseRazerFX):
     def __init__(self, serial: str, capabilities: dict, daemon_dbus=None, matrix_dims=(-1, -1)):
-        super(RazerAdvancedFX, self).__init__(serial, capabilities, daemon_dbus)
+        super().__init__(serial, capabilities, daemon_dbus)
 
         # Only init'd when there's a matrix
         self._capabilities = capabilities
@@ -612,10 +652,16 @@ class RazerAdvancedFX(BaseRazerFX):
             else:
                 raise ValueError("RGB must be an RGB tuple")
 
+    def restore(self):
+        """
+        Restore the device to the last effect
+        """
+        self._lighting_dbus.restoreLastEffect()
+
 
 class SingleLed(BaseRazerFX):
     def __init__(self, serial: str, capabilities: dict, daemon_dbus=None, led_name='logo'):
-        super(SingleLed, self).__init__(serial, capabilities, daemon_dbus)
+        super().__init__(serial, capabilities, daemon_dbus)
 
         self._led_name = led_name
         self._lighting_dbus = _dbus.Interface(self._dbus, "razer.device.lighting.{0}".format(led_name))
@@ -624,7 +670,7 @@ class SingleLed(BaseRazerFX):
         return self.has('{0}_{1}'.format(self._led_name, item))
 
     def _getattr(self, name):
-        attr = name.replace('#', self._led_name.title())
+        attr = name.replace('#', self._led_name.title().replace("_", ""))
         return getattr(self._lighting_dbus, attr, None)
 
     @property
@@ -644,6 +690,46 @@ class SingleLed(BaseRazerFX):
             else:
                 func(False)
 
+    @property
+    def effect(self) -> str:
+        """
+        Get current effect
+
+        :return: Effect name ("static", "spectrum", etc.)
+        :rtype: str
+        """
+        return str(self._getattr('get#Effect')())
+
+    @property
+    def colors(self) -> bytearray:
+        """
+        Get current effect colors
+
+        :return: Effect colors (an array of 9 bytes, for 3 colors in RGB format)
+        :rtype: bytearray
+        """
+        return bytes(self._getattr('get#EffectColors')())
+
+    @property
+    def speed(self) -> int:
+        """
+        Get current effect speed
+
+        :return: Effect speed (a value between 0 and 3)
+        :rtype: int
+        """
+        return int(self._getattr('get#EffectSpeed')())
+
+    @property
+    def wave_dir(self) -> int:
+        """
+        Get current wave direction
+
+        :return: Wave direction (WAVE_LEFT or WAVE_RIGHT)
+        :rtype: int
+        """
+        return int(self._getattr('get#WaveDir')())
+
     @property
     def brightness(self):
         if self._shas('brightness'):
@@ -889,7 +975,7 @@ class SingleLed(BaseRazerFX):
 
 class MiscLighting(BaseRazerFX):
     def __init__(self, serial: str, capabilities: dict, daemon_dbus=None):
-        super(MiscLighting, self).__init__(serial, capabilities, daemon_dbus)
+        super().__init__(serial, capabilities, daemon_dbus)
 
         self._lighting_dbus = _dbus.Interface(self._dbus, "razer.device.lighting.logo")
 
@@ -913,6 +999,21 @@ class MiscLighting(BaseRazerFX):
         else:
             self._right = None
 
+        if self.has('charging'):
+            self._charging = SingleLed(serial, capabilities, daemon_dbus, 'charging')
+        else:
+            self._charging = None
+
+        if self.has('fast_charging'):
+            self._fast_charging = SingleLed(serial, capabilities, daemon_dbus, 'fast_charging')
+        else:
+            self._fast_charging = None
+
+        if self.has('fully_charged'):
+            self._fully_charged = SingleLed(serial, capabilities, daemon_dbus, 'fully_charged')
+        else:
+            self._fully_charged = None
+
         if self.has('backlight'):
             self._backlight = SingleLed(serial, capabilities, daemon_dbus, 'backlight')
         else:
@@ -934,6 +1035,18 @@ class MiscLighting(BaseRazerFX):
     def right(self):
         return self._right
 
+    @property
+    def charging(self):
+        return self._charging
+
+    @property
+    def fast_charging(self):
+        return self._fast_charging
+
+    @property
+    def fully_charged(self):
+        return self._fully_charged
+
     @property
     def backlight(self):
         return self._backlight
